// grapheditDoc.h : interface of the CGrapheditDoc class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_GRAPHEDITDOC_H__5369F2CC_F096_11D4_9E24_0080C88B30AF__INCLUDED_)
#define AFX_GRAPHEDITDOC_H__5369F2CC_F096_11D4_9E24_0080C88B30AF__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

//#include <vector>

template<class T> class svector
{
public:
	typedef T * iterator;

	inline svector() 
	{
		buf =0;
		siz =0;
		mem =0; 
	}
	inline svector( const int n )
	{
		siz = mem = n;
		buf  = new T[n];
	}
	inline ~svector()
	{
		if(buf) delete[] buf;
	}
	inline T & operator[]( const int i )
	{
		return buf[i];
	}
	inline const T & operator[]( const int i ) const
	{
		return buf[i];
	}
	inline int size() const { return siz; }
	inline void alloc( int n )
	{
		if(n>=mem)
		{
			if(mem==0)
			{
				mem = n;
				buf = new T[n];
			}
			else
			{
				T * t = new T[n];
				for(int i=0;i<mem;++i)
					t[i]=buf[i];
				delete[] buf;
				buf = t;
				mem = n;
			}
		}
	}
	inline svector & resize( const int n )
	{
		if(n>=mem) alloc(n);
		siz = n;
		return *this;
	}

	inline svector & push_back( const T & a )
	{
		if(siz>=mem)
		{
			if(mem==0) alloc(256);
			else       alloc(mem*2);
		}
		buf[siz++]=a;
		return *this;
	}
	inline iterator begin() { return buf; }
	inline iterator end() { return buf+siz; }
private:
	T * buf;
	int siz;
	int mem;
};

class Node
{
public:
	CPoint p;

	inline Node() {}
	inline Node( const CPoint & np ) { p = np; }
	void Serialize(CArchive& ar);
};

class Arco
{
public:
	int n[2];

	inline Arco() {}
	inline Arco( const int n0, const int n1 ) { n[0]=n0; n[1]=n1; }
	inline bool operator== ( const Arco & a )
	{
		return n[0]==a.n[0] && n[1]==a.n[1];
	}
	void Serialize(CArchive& ar);
};


class CGrapheditDoc : public CDocument
{
protected: // create from serialization only
	CGrapheditDoc();
	DECLARE_DYNCREATE(CGrapheditDoc)

// Attributes
public:
	typedef svector<Node>::iterator node_iterator;
	typedef svector<Arco>::iterator  arc_iterator;

	svector<Node> nodes;
	svector<Arco>  arcs;

// Operations
public:

	inline void add_node( const CPoint & p )
	{
		nodes.push_back( Node(p) );
		UpdateAllViews(0);
	}

	inline void add_arc( const int n0, const int n1 )
	{
		arcs.push_back( Arco(n0,n1) );
		UpdateAllViews(0);
	}

	int FindNode( const CPoint & p );

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CGrapheditDoc)
	public:
	virtual BOOL OnNewDocument();
	virtual void Serialize(CArchive& ar);
	virtual BOOL OnSaveDocument(LPCTSTR lpszPathName);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CGrapheditDoc();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	//{{AFX_MSG(CGrapheditDoc)
		// NOTE - the ClassWizard will add and remove member functions here.
		//    DO NOT EDIT what you see in these blocks of generated code !
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_GRAPHEDITDOC_H__5369F2CC_F096_11D4_9E24_0080C88B30AF__INCLUDED_)
